/***
 * Juliac
 * Copyright (C) 2009-2013 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 * Contributor: Romain Rouvoy
 */

package org.objectweb.fractal.juliac.osgi;

import java.lang.reflect.Modifier;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.Utils;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.proxy.AbstractProxyClassGenerator;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.juliac.visit.BlockSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.ClassSourceCodeVisitor;

/**
 * This class generates a stub for transmitting service invocations from the
 * OSGi world to the Fractal world. The generated stub exports an OSGi service
 * which is imported by the OSGi bundle hosted by the osgiPrimitive membrane.
 * When receiving service invocations from the hosted OSGi bundle, the generated
 * stub delegates the call to the corresponding Fractal client interface using
 * dynamic Java method calls.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @author Romain Rouvoy <Romain.Rouvoy@univ-lille1.fr>
 * @since 2.1.6
 */
public class DynServiceClassGenerator extends AbstractProxyClassGenerator {

	private DynProxyGenerator dpg = new DynProxyGenerator();
	
    public DynServiceClassGenerator(
        Juliac jc,
        InterfaceType it, MembraneDesc<?> membraneDesc, boolean mergeable ) {
        
    	super(jc,it,membraneDesc,mergeable);
    }

    /**
     * Return <code>true</code> if some code must be generated by the current
     * source class generator.
     */
    public boolean match() {
        return true;
    }
    
    public String getDelegatingInstance( UnifiedClass proxycl, UnifiedMethod proxym ) {
        return "_impl";
    }

    /**
     * Return the name of the class generated by this generator.
     */
    @Override
    public String getTargetClassName() {
        String javaItfSignature = it.getFcItfSignature();
        String name = Utils.getJuliacGeneratedStrongTypeName(javaItfSignature);
        String suffix = getClassNameSuffix();
        String targetClassName = name + suffix;
        String hexhash = Integer.toHexString(targetClassName.hashCode());
        String osgiTargetClassName = "osgi"+hexhash+"."+targetClassName;
        String rootedTargetClassName = jc.getRootedClassName(osgiTargetClassName);
        return rootedTargetClassName;
    }

    public String getClassNameSuffix() {
        return "Dyn";
    }
    
    @Override
    public String[] getImplementedInterfaceNames() {
        String javaItfSignature = it.getFcItfSignature();
        return new String[]{javaItfSignature};
    }
    
    @Override
    public void generateFields( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {
        
        cv.visitField( Modifier.PRIVATE, "java.util.Map", "methodsMap", "new java.util.HashMap()" );
        cv.visitField( Modifier.PRIVATE, "Object", "_impl", null );

        super.generateFields(cv,proxycl);
    }
        
    @Override
    public void generateMethods( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {

        // Method setFcItfDelegate
        BlockSourceCodeVisitor mv =
            cv.visitMethod(
                Modifier.PUBLIC, null, "void", "setFcItfDelegate",
                new String[]{"Object obj"}, null );
        
        dpg.generateMethodSetFcItfDelegate(mv,proxycl,"_impl");
        mv.visitEnd();
    }

    @Override
    public void generateProxyMethodBodyDelegatingCode(
        BlockSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
        
        dpg.generateProxyMethodBodyDelegatingCode(mv,proxycl,proxym,"_impl");
    }
}
