/***
 * Juliac
 * Copyright (C) 2009-2013 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.osgi;

import java.io.IOException;
import java.util.List;

import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.desc.ControllerDesc;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.opt.InitializerClassGenerator;
import org.objectweb.fractal.juliac.proxy.InterceptorClassGenerator;
import org.objectweb.fractal.juliac.proxy.InterceptorSourceCodeGeneratorItf;
import org.objectweb.fractal.juliac.proxy.LifeCycleSourceCodeGenerator;
import org.objectweb.fractal.juliac.proxy.ProxyClassGeneratorItf;

/**
 * Source code generator for osgiPrimitive components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 2.1.5
 */
public class FCOOCtrlSourceCodeGenerator
extends org.objectweb.fractal.juliac.opt.oo.FCOOCtrlSourceCodeGenerator {
    
	private FCOOCtrlGenerator osgig;
	
    /** Configuration file for OSGi. */
    public static final String DEFAULT_CONFIGURATION = "osgi-bundled.cfg";

    
    // -----------------------------------------------------------------------
    // Implementation of the JuliacModuleItf interface
    // -----------------------------------------------------------------------
    
    @Override
    public void init( Juliac jc ) throws IOException {
    	super.init(jc);
        osgig = new FCOOCtrlGenerator(jc);
    }

    @Override
    protected void postInitJuliaLoaderModule() throws IOException {
        jloader.loadConfigFile(DEFAULT_CONFIGURATION);
        super.postInitJuliaLoaderModule();
    }

    
    // -----------------------------------------------------------------------
    // Implementation of the FCSourceCodeGeneratorItf interface
    // -----------------------------------------------------------------------

    /**
     * Return <code>true</code> if the current source code generator handles the
     * specified controller descriptor.
     * 
     * @param controllerDesc  the controller descriptor to be checked
     * @return                if <code>controllerDesc</code> should be included 
     * @since 2.2
     */
    @Override
    public boolean acceptCtrlDesc( Object controllerDesc ) {
        boolean accept = controllerDesc.equals("osgiPrimitive");
        return accept;
    }

    /**
     * Generate the source code associated to a Fractal component.
     * The membrane implementation and the initializer implementation are
     * generated.
     * 
     * @param type            the component type
     * @param controllerDesc  the component controller descriptor
     * @param contentDesc     the component content descriptor
     */
    @Override
    public InitializerClassGenerator<List<ControllerDesc>> generate(
        Type type, Object controllerDesc, Object contentDesc )
    throws IOException {
        
        /*
         * Check parameter consistence.
         */
    	osgig.check(type,controllerDesc,contentDesc);

    	InitializerClassGenerator<List<ControllerDesc>> scg =
            super.generate(type,controllerDesc,contentDesc);
        return scg;
    }
    
    /**
     * Return the source code generators for the initializer classes associated
     * with this component source code generator.
     */
    @Override
    public InitializerOOCtrlClassGenerator getInitializerClassGenerator() {
        return new InitializerOOCtrlClassGenerator();
    }
    
    /**
     * Return the interceptor class generator for the specified controller
     * descriptor. Return <code>null</code> if the specified controller
     * descriptor is not associated with an interceptor source code generator.
     * 
     * @since 2.2.5
     */
    @Override
    protected ProxyClassGeneratorItf getInterceptorClassGenerator(
        ComponentType ct, String ctrldesc ) {
        
    	boolean mergeable = false;
    	InterceptorSourceCodeGeneratorItf lcscg =
    		new LifeCycleSourceCodeGenerator();
    	lcscg.setJuliac(jc);
    	lcscg.setMergeable(mergeable);

    	InterceptorSourceCodeGeneratorItf[] iscgs =
    		new InterceptorSourceCodeGeneratorItf[]{lcscg};

    	InterceptorClassGenerator itbcg = new DynInterceptorClassGenerator();
    	itbcg.setJuliac(jc);
    	itbcg.init(iscgs);
    	return itbcg;
    }
    
    /**
     * Return the source code generator for component interfaces.
     */
    @Override
    public ProxyClassGeneratorItf getInterfaceClassGenerator( 
        InterfaceType it, MembraneDesc<?> membraneDesc ) {

    	ProxyClassGeneratorItf pcg =
    		osgig.getInterfaceClassGenerator(it,membraneDesc);
    	return pcg;
    }

    
    // ------------------------------------------------------------------
    // Implementation of methods defined in FCSourceCodeGenerator
    // ------------------------------------------------------------------
    
    @Override
    protected void generateInterfaceImpl( InterfaceType[] its, String ctrlDesc )
    throws IOException {
        
        for (InterfaceType it : its) {
            
            generateInterfaceImpl(it,ctrlDesc);
            
            /*
             * Super method generate internal interfaces. We don't need them for
             * osgiPrimitive.
             */
        }
    }
    
    /**
     * Generate the source code of component interfaces associated with the
     * specified interface type.
     */
    @Override
    protected void generateInterfaceImpl( InterfaceType it, String ctrlDesc )
    throws IOException {
    	
        osgig.generateInterfaceImpl(it,ctrlDesc);
    }    
}
